﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using FCS.Common;
using FCS.Crawler.ZCLotteryMatchs;
using FCS.Interface;
using FCS.Models;
using FCS.Models.Entity;

namespace FCS.Crawler.Basketball
{
    public class B_GamesJob : CommonJob
    {
        private Dictionary<string, string> statusDict_NBA;//页面与数据库状态对应枚举NBA
        private List<DataItemDetail> gamesStatus;//比赛状态
        private List<B_Team> teamList;//球员
        private List<B_Games> result = new List<B_Games>();
        private string eventId;
        private List<B_Grouping> groupList = new List<B_Grouping>();

        public void Click()
        {
            CommonHelper.LogBD(typeof(B_GamesJob), () =>
            {
                Click_NBA();
                Click_CBA();
            });
         
        }
        #region NBA
        /// <summary>
        /// 获取全部
        /// </summary>
        private void GetALL_NBA()
        {
            GetSqlData();
            //季前赛 http://lanqiu.zgzcw.com/nba/saichengbeforebyteam.action?source_league_id=1&season=17-18
            //常规赛 http://lanqiu.zgzcw.com/nba/SaichengMonthTeamAjax.action?source_league_id=1&season=17-18&teamId=&guestTeamId=&month=03
            //季后赛 http://lanqiu.zgzcw.com/nba/saichengafterByTeam.action?source_league_id=1&season=17-18&teamId=0&guestTeamId=0&group=1
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["NBAEventName"]).ToList()[0].Id;
            var groupList = services.Query<B_Grouping>(" AND EventId='{0}'".FormatMe(eventId)).ToList();
            groupList.ToList().ForEach(p =>
             {
                 Analysis_NBA(p);
             });
            while (true)
            {
                if (CommonHelper.ThreadsFinsh())
                    break;
            }
            services.SqlBulkCopyAdd(result);
        }

        /// <summary>
        /// 更新
        /// </summary>
        private void Click_NBA()
        {
            GetSqlData();
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["NBAEventName"]).ToList()[0].Id;
            var groupList = services.Query<B_Grouping>(" AND Season LIKE '%{0}%' AND EventId='{1}'".FormatMe(DateTime.Now.Year.ToString().Substring(2), eventId)).ToList();
            groupList.ForEach(p =>
              {
                  Analysis_NBA(p);
              });
            Task.WaitAll(taskList.ToArray());
            var deleteWhere = "";
            groupList.ForEach(p =>
            {
                deleteWhere += "'" + p.Id + "',";

            });
            deleteWhere = deleteWhere.ToString().Substring(0, deleteWhere.ToString().Length - 1);
            services.Delete<B_Games>(" AND GroupingId IN ({0})".FormatMe(deleteWhere));
            services.SqlBulkCopyAdd(result);
        }

        /// <summary>
        /// 获取Html分析出数据
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        private async Task Analysis_NBA(B_Grouping p)
        {
            taskList.Add(Task.Run(() =>
            {

                var url = "";
                if (p.Extended1.IsEmpty() || p.Extended1 == "0")
                    return;
                if (p.Extended1 == "-1")
                    url = "http://lanqiu.zgzcw.com/nba/saichengbeforebyteam.action?source_league_id=1&teamId=0&guestTeamId=0&season=" + p.Season;
                else if (p.Extended1.Length == 1)
                    url = "http://lanqiu.zgzcw.com/nba/saichengafterByTeam.action?source_league_id=1&season=" + p.Season + "&teamId=&guestTeamId=&group=" + p.Extended1;
                else
                    url = "http://lanqiu.zgzcw.com/nba/SaichengMonthTeamAjax.action?source_league_id=1&season=" + p.Season + "&teamId=&guestTeamId=&month=" + p.Extended1;

                var doc = CommonHelper.GetHtml(url);
                var gamesList = doc.DocumentNode.SelectNodes(".//tr");
                var number = 0;
                while (doc.DocumentNode.InnerHtml.Contains("html") || gamesList == null)
                {
                    if (number > 10)
                        break;
                    if (!doc.DocumentNode.InnerHtml.Contains("html"))
                        number++;
                    doc = CommonHelper.GetHtml(url);
                }
                if (gamesList == null)
                    return;
                foreach (var item in gamesList)
                {
                    var td = item.SelectNodes("td");
                    if (td == null || td.Count < 3 || td[2].SelectNodes(".//strong") == null)
                        break;
                    var scoreAll = td[2].SelectNodes(".//strong")[0].InnerText;
                    result.Add(new B_Games
                    {
                        Id = CommonHelper.GetGuid().ToString(),
                        HomeTeamId = string.Join(",", (from a in teamList
                                                       where a.Name == td[1].InnerText
                                                       select a.Id).ToArray()),
                        VisitingTeamId = string.Join(",", (from a in teamList
                                                           where a.Name == td[3].InnerText
                                                           select a.Id).ToArray()),
                        HomeTeamScore_All = scoreAll.IsEmpty() || scoreAll.Trim().Contains("-") ? 0
                         : Convert.ToInt32(scoreAll.Trim().Split(':')[0]),
                        VisitingTeamScore_All = scoreAll.IsEmpty() || scoreAll.Trim().Contains("-") ? 0
                         : Convert.ToInt32(scoreAll.Trim().Split(':')[1]),
                        HomeTeamScore_Half = td[5].InnerText.IsEmpty() ? 0 : Convert.ToInt32(td[5].InnerText.Trim().Split(':')[0]),
                        VisitingTeamScore_Half = td[5].InnerText.IsEmpty() ? 0 : Convert.ToInt32(td[5].InnerText.Trim().Split(':')[1]),
                        Status = td[4].InnerText.Trim().IsEmpty() ? gamesStatus.Where(q => q.ItemValue == "1").ToList()[0].Id : statusDict_NBA[td[4].InnerText.Trim()],
                        Remark = td[4].InnerText.Trim(),
                        EventId = eventId,
                        CreateDateTime = DateTime.Now,
                        GroupingId = p.Id,
                        Season = p.Season,
                        StartDateTime = DateTime.Parse("20" + p.Season.Split('-')[1] + "-" + td[0].InnerText.Trim())
                    });
                }
            }));
        }
        #endregion

        #region CBA
        /// <summary>
        /// 更新
        /// </summary>
        private void Click_CBA()
        {
            GetSqlData();
            result = new List<B_Games>();
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["CBAEventName"]).ToList()[0].Id;
            groupList = services.Query<B_Grouping>(" AND Season LIKE '%{0}%' AND EventId='{1}'".FormatMe(DateTime.Now.Year.ToString().Substring(2), eventId)).ToList();
            var groupParentList = groupList.Where(p => string.IsNullOrEmpty(p.ParentId)).ToList();
            groupParentList.ForEach(p =>
            {
                Analysis_CBA(p);
            });
            Task.WaitAll(taskList.ToArray());
            var deleteWhere = "";
            groupList.ForEach(p =>
            {
                deleteWhere += "'" + p.Id + "',";

            });
            deleteWhere = deleteWhere.ToString().Substring(0, deleteWhere.ToString().Length - 1);
            services.Delete<B_Games>(" AND GroupingId IN ({0})".FormatMe(deleteWhere));
            services.SqlBulkCopyAdd(result);

        }


        private void GetALL_CBA()
        {
            GetSqlData();
            result = new List<B_Games>();
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["CBAEventName"]).ToList()[0].Id;
            groupList = services.Query<B_Grouping>(" AND EventId='{0}'".FormatMe(eventId)).ToList();
            var groupParentList = groupList.Where(p => string.IsNullOrEmpty(p.ParentId)).ToList();
            groupParentList.ForEach(p =>
            {
                Analysis_CBA(p);
            });
            Task.WaitAll(taskList.ToArray());
            services.SqlBulkCopyAdd(result);
        }
        private async Task Analysis_CBA(B_Grouping p)
        {
            taskList.Add(Task.Run(() =>
             {
                 if (p.Extended1 == "0")
                     return;

                 if (p.Name.Contains("季前赛"))
                 {
                     var html = new B_GroupingJob().GetPageHtml(p.Remark, p.Season);
                     AddData(html, p, p.Remark);
                 }
                 else if (p.Name.Contains("常规赛"))
                 {
                     var groupChlidList = groupList.Where(q => q.ParentId == p.Id && q.Extended1 != "0").ToList();
                     groupChlidList.ForEach(s =>
                     {
                         var url = p.Remark + $"&y={s.Extended1}&m={s.Name}";
                         var html = new B_GroupingJob().GetPageHtml(url, s.Season);
                         AddData(html, s, url);
                     });
                 }
                 else
                 {
                     var html = new B_GroupingJob().GetPageHtml(p.Remark, p.Season);
                     var playoffsList = Regex.Split(html, "playoffsList")[1].Split(';')[0].Replace("=", "").Trim().JsonToList<string[]>();//分组数据
                     var pfData = Regex.Split(html, "pfData");
                     foreach (var item in playoffsList)//比赛数据为多个pfData，，需要循环新增比赛到相应的分组
                     {
                         var code = item[0];
                         foreach (var pfItem in pfData)
                         {
                             var data = Regex.Split(pfItem, "P" + code);
                             if (data.Count() > 1)
                             {
                                 var pageTeamData = new List<List<object>>();
                                 var pageTeamDataAll = data[1].Split(';')[0].Split('=')[1].Trim().JsonToList<List<object>>();
                                 pageTeamDataAll.ForEach(s =>
                                 {
                                     pageTeamData = pageTeamData.Concat(s[4].ToString().Replace("{", "").Replace("}", "").Trim().JsonToList<List<object>>()).ToList();
                                 });
                                 var groupData = groupList.Where(s => s.Name == item[1] && s.Season == p.Season).ToList()[0];//相应数据库分组数据
                                 AddData(html, groupData, p.Remark, pageTeamData);
                             }
                         }
                     }

                 }
             }));
        }

        private void AddData(string html, B_Grouping p, string url, List<List<object>> data = null)
        {
            if (data.IsEmpty())
            {
                var arrData = Regex.Split(html, "arrData")[1].Split(';')[0].Replace("=", "").Trim();
                data = arrData.JsonToList<List<object>>();
            }
            //获取球队信息
            var arrTeam = Regex.Split(html, "arrTeam")[1].Split(';')[0].Replace("=", "").Trim();
            var pageTeamData = arrTeam.JsonToList<List<string>>();

            foreach (var item in data)
            {
                result.Add(new B_Games
                {
                    Id = CommonHelper.GetGuid().ToString(),
                    CreateDateTime = DateTime.Now,
                    GroupingId = p.Id,
                    Season = p.Season,
                    Remark = (item[5].TryToInt32() > 0 && item[7].TryToInt32() > 0) ? "胜" : "负",
                    StartDateTime = item[2].TryToDateTime(),
                    HomeTeamId = string.Join(",", (from a in teamList
                                                   where a.Name == GetTeamName(pageTeamData, item[3].ToString())
                                                   select a.Id).ToArray()),
                    VisitingTeamId = string.Join(",", (from a in teamList
                                                       where a.Name == GetTeamName(pageTeamData, item[4].ToString())
                                                       select a.Id).ToArray()),
                    EventId = eventId,
                    HomeTeamScore_All = item[5].TryToInt32(),
                    VisitingTeamScore_All = item[6].TryToInt32(),
                    HomeTeamScore_Half = item[7].TryToInt32(),
                    VisitingTeamScore_Half = item[8].TryToInt32(),
                    Status = (item[5].TryToInt32() > 0 && item[7].TryToInt32() > 0) ?
                             gamesStatus.Where(q => q.ItemValue == "4").ToList()[0].Id :
                             gamesStatus.Where(q => q.ItemValue == "1").ToList()[0].Id,

                });

            }
        }

        /// <summary>
        /// 获取页面球队名称
        /// </summary>
        /// <param name="data"></param>
        /// <param name="code"></param>
        /// <returns></returns>
        public string GetTeamName(List<List<string>> data, string code)
        {
            foreach (var item in data)
            {
                var list = item.Where(p => p == code).ToList();
                if (list.Count > 0)
                    return item[1];
            }
            return string.Empty;
        }
        #endregion

        private void GetSqlData()
        {
            gamesStatus = services.GetDataItem(DataItemDetailEnum.FootBallGameStatus);
            teamList = services.Query<B_Team>().ToList();
            var finshStatus = gamesStatus.Where(p => p.ItemValue == "4").ToList()[0].Id;
            statusDict_NBA = new Dictionary<string, string> {
                {"胜", finshStatus },
                {"负",finshStatus}
            };
        }
    }
}