﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using FCS.Crawler.ZCLotteryMatchs;
using FCS.Interface;
using FCS.Common;
using FCS.Models.Entity;
using HtmlAgilityPack;
using System.Threading;
using Newtonsoft.Json;
using System.Net;
using System.Text.RegularExpressions;

namespace FCS.Crawler.Basketball
{
    public class B_GroupingJob : CommonJob
    {
        private string eventId = "";
        private List<B_Grouping> result = new List<B_Grouping>();

        public void Click()
        {
            CommonHelper.LogBD(typeof(B_GroupingJob), () =>
            {
                Click_NBA();
                Click_CBA();
            });
        }

        #region NBA
        private void Click_NBA()
        {
            result = new List<B_Grouping>();
            var url = "http://lanqiu.zgzcw.com/1/{0}/saiAll.do".FormatMe(
               DateTime.Now.AddYears(-1).Year.ToString().Substring(2) + "-" + DateTime.Now.Year.ToString().Substring(2));
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["NBAEventName"]).ToList()[0].Id;
            var doc = CommonHelper.GetHtml(url, "足彩");//加载主页面
            var season = GetNBASeason(doc.DocumentNode).ToList();
            foreach (var model in season)
            {
                taskList.Add(Task.Run(() =>
                {
                    if (model.Remark != url)
                        doc = CommonHelper.GetHtml(model.Remark, "足彩");//加载主页面
                    var parentGroup = doc.DocumentNode.SelectSingleNode(".//div[@attr='saiAll']").SelectNodes("./span");
                    foreach (var item in parentGroup)
                    {
                        var childId = item.Attributes["Id"].Value.Replace("Title", "") + "s";
                        var childGroup_Div = doc.DocumentNode.SelectSingleNode(".//div[@id='{0}']".FormatMe(childId)).SelectSingleNode(".//div[@class='xq']");

                        var id = CommonHelper.GetGuid().ToString();
                        result.Add(GetModel(new B_Grouping
                        {
                            Id = id,
                            Name = item.SelectNodes("./a")[0].InnerText.Trim(),
                            Sort = parentGroup.IndexOf(item) + 1,
                            Remark = model.Remark,
                            Season = model.Season,
                            Extended1 = childGroup_Div == null ? "-1" : string.Empty,
                        }));
                        if (childGroup_Div != null)
                        {
                            var childGroup = childGroup_Div.SelectNodes(".//a");
                            if (childGroup != null)
                            {
                                foreach (var _item in childGroup)
                                {
                                    result.Add(GetModel(new B_Grouping
                                    {
                                        Name = _item.InnerText.Trim(),
                                        Sort = childGroup.IndexOf(_item) + 1,
                                        ParentId = id,
                                        Remark = model.Remark,
                                        Season = model.Season,
                                        Extended1 = _item.Attributes["attr"].Value
                                    }));
                                }
                            }

                        }
                    }

                }));
            }
            Task.WaitAll(taskList.ToArray());
            AddData();
        }
        private B_Grouping GetModel(B_Grouping model)
        {
            if (model.Id.IsEmpty())
                model.Id = CommonHelper.GetGuid().ToString();
            model.CreateDateTime = DateTime.Now;
            model.EventId = eventId;

            return model;
        }

        /// <summary>
        /// 得到NBA的期数
        /// </summary>
        /// <param name="doc"></param>
        /// <returns></returns>
        private IEnumerable<B_Grouping> GetNBASeason(HtmlNode doc)
        {
            // var list = new  List< B_Grouping >()
            var oldSeason = (from a in services.Query<B_Grouping>(" AND EventId='{0}'".FormatMe(eventId))
                             group a by a.Season into g
                             select g.Key).ToList();

            var season = doc.SelectSingleNode(".//select[@id='selectSeason']").SelectNodes(".//option");
            foreach (var item in season)
            {
                if (!oldSeason.Contains(item.Attributes["value"].Value) || item.Attributes["value"].Value.Contains(DateTime.Now.Year.ToString()))
                    yield return new B_Grouping
                    {
                        Remark = "http://lanqiu.zgzcw.com/1/" + item.Attributes["value"].Value + "/saiAll.do",
                        Season = item.Attributes["value"].Value
                    };
            }
        }
        #endregion

        #region CBA
        public void Click_CBA()
        {
            result = new List<B_Grouping>();
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["CBAEventName"]).ToList()[0].Id;
            var seasonList = GetCBASeason().OrderByDescending(p => p.Season).ToList();
            seasonList.ForEach(p =>
            {
                taskList.Add(Task.Run(() =>
                {
                    var url = $"http://nba.nowscore.com/cn/Normal.aspx?SclassID=5&matchSeason={p.Season}";//常规赛
                    var html = GetPageHtml(url, p.Season);
                    var ymList = Regex.Split(html, "ymList")[1].Split(';')[0].Replace("=", "").Trim().JsonToEntity<string[][]>();
                    var id = CommonHelper.GetGuid().ToString();
                    result.Add(GetModel(new B_Grouping
                    {
                        Id = id,
                        Name = "常规赛",
                        Sort = 1,
                        Remark = url,
                        Season = p.Season,
                    }));
                    var i = 0;
                    foreach (var item in ymList)
                    {
                        var list = result.Where(q => q.Name == item[0] && q.Season == p.Season).ToList();
                        if (list.Count == 0)
                        {
                            result.Add(GetModel(new B_Grouping
                            {
                                ParentId = id,
                                Name = item[0],
                                Sort = 0,
                                Remark = url,
                                Season = p.Season,
                                Extended1 = "0"
                            }));
                        }
                        result.Add(GetModel(new B_Grouping
                        {
                            Name = item[1],
                            Sort = i,
                            ParentId = id,
                            Remark = url,
                            Season = p.Season,
                            Extended1 = item[0]
                        }));
                        i++;
                    }
                    var arrLeague = html.Split(';')[0].Split('=')[1].JsonToList<string>();
                    if (arrLeague[arrLeague.Count - 1].Contains("3"))
                    {
                        id = CommonHelper.GetGuid().ToString();
                        result.Add(GetModel(new B_Grouping
                        {
                            Id = id,
                            Name = "季前赛",
                            Sort = 0,
                            Remark = $"http://nba.nowscore.com/cn/Preseason.aspx?SclassID=5&matchSeason={p.Season}",
                            Season = p.Season
                        }));
                    }
                    if (arrLeague[arrLeague.Count - 1].Contains("2"))
                    {
                        //http://nba.nowscore.com/cn/Playoffs.aspx?matchSeason=2016-2017&SclassID=5
                        url = $"http://nba.nowscore.com/cn/Playoffs.aspx?SclassID=5&matchSeason={p.Season}";
                        id = CommonHelper.GetGuid().ToString();
                        result.Add(GetModel(new B_Grouping
                        {
                            Id = id,
                            Name = "季后赛",
                            Sort = 2,
                            Remark = url,
                            Season = p.Season
                        }));
                        html = GetPageHtml(url, p.Season);
                        var playoffsList = Regex.Split(html, "playoffsList")[1].Split(';')[0].Replace("=", "").Trim().JsonToList<string[]>();
                        foreach (var item in playoffsList)
                        {
                            result.Add(GetModel(new B_Grouping
                            {
                                Name = item[1],
                                Sort = playoffsList.IndexOf(item),
                                ParentId = id,
                                Remark = url,
                                Season = p.Season,
                                Extended1 = "1"
                            }));
                        }
                    }

                }));
            });
            Task.WaitAll(taskList.ToArray());
            AddData();
        }

        public string GetPageHtml(string url, string season)
        {

            var doc = CommonHelper.GetHtmlHtmlDocument(new Models.DTO.HtmlParameterDTO { Url = url, Allowautoredirect = true, Timeout = 10 * 1000 ,Title= "捷报比分" });
            var scripts = doc.DocumentNode.SelectNodes(".//script");
            foreach (var item in scripts)
            {
                if (!item.Attributes["src"].IsEmpty() && item.Attributes["src"].Value.Contains("jsData") && item.Attributes["src"].Value.Contains(season.Replace("20", "")))
                {
                    url = item.Attributes["src"].Value;
                    break;
                }
            }
            var html = CommonHelper.GetHtmlString(new Models.DTO.HtmlParameterDTO { Url = "http://nba.nowscore.com" + url, Allowautoredirect = true, Title = "lastUpdateTime", Timeout = 10 * 1000 });
            while (html.ToLower().Contains("html"))
                html = CommonHelper.GetHtmlString(new Models.DTO.HtmlParameterDTO { Url = "http://nba.nowscore.com" + url, Allowautoredirect = true, Title = "lastUpdateTime", Timeout = 10 * 1000 });
            return html;

        }

        /// <summary>
        /// 得到NBA的期数
        /// </summary>
        /// <param name="doc"></param>
        /// <returns></returns>
        private IEnumerable<B_Grouping> GetCBASeason()
        {
            var oldSeason = (from a in services.Query<B_Grouping>(" AND EventId='{0}'".FormatMe(eventId))
                             group a by a.Season into g
                             select g.Key).ToList();
            var url = "http://nba.nowscore.com/jsData/LeagueSeason/sea5.js";



            var html = CommonHelper.GetHtmlString(new Models.DTO.HtmlParameterDTO { Url = url, Allowautoredirect = true });
            var number = 0;
            while (html.ToLower().Contains("html"))
            {
                if (number > 10)
                    break;
                html = CommonHelper.GetHtmlString(new Models.DTO.HtmlParameterDTO { Url = url, Allowautoredirect = true });
                number++;
            }
            var data = JsonConvert.DeserializeObject<string[][]>(html.Split('=')[1].Replace(";", ""));
            foreach (var item in data)
            {

                if (!oldSeason.Contains(item[0]) || item[0].Contains(DateTime.Now.Year.ToString()))
                    yield return new B_Grouping
                    {
                        Season = item[0].ToString(),
                        Remark = item[1].ToString(),
                    };
            }
        }
        #endregion

        /// <summary>
        /// 新增数据
        /// </summary>
        private void AddData()
        {
            var data = new List<B_Grouping>();
            var groupList = services.Query<B_Grouping>(" AND EventId='{0}'".FormatMe(eventId));
            //新增的父分组、子分组
            var parentList = (from a in result
                              where !(from b in groupList where string.IsNullOrEmpty(b.ParentId) && a.EventId == b.EventId select b.Season.Trim()).Contains(a.Season.Trim())
                              && string.IsNullOrEmpty(a.ParentId)
                              select a).ToList();
            parentList = parentList.Concat((from a in result
                                            where !(from b in groupList where string.IsNullOrEmpty(b.ParentId) && a.EventId == b.EventId && a.Season == b.Season select b.Name.Trim()).Contains(a.Name.Trim())
                                            && string.IsNullOrEmpty(a.ParentId)
                                            select a).ToList()).Distinct().ToList();

            parentList.ForEach(p =>
            {
                data.Add(p);
                var childList = result.Where(q => q.ParentId == p.Id).ToList();
                childList.ForEach(q =>
                {
                    data.Add(q);
                });
            });

            //新增的子分组
            groupList.Where(p => string.IsNullOrEmpty(p.ParentId)).ToList().ForEach(q =>
            {
                var oldList = groupList.Where(s => (s.ParentId.IsEmpty() ? string.Empty : s.ParentId.Trim()) == q.Id && s.EventId == q.EventId).ToList();//数据库子节点的数据
                var newParentId = result.Where(s => s.Name.Trim() == q.Name.Trim() && s.Season == q.Season && s.EventId == q.EventId && string.IsNullOrEmpty(s.ParentId)).ToList();//爬取数据子节点数据
                var newList = result.Where(s => s.ParentId == (newParentId.Count > 0 ? newParentId[0].Id : "0")).ToList();
                if (newList.Count > 0 && oldList.Count != newList.Count)
                {
                    var list = (from a in newList
                                where !(from b in oldList select b.Name.Trim()).Contains(a.Name.Trim())
                                select a).ToList();
                    list.ForEach(d =>
                    {
                        d.ParentId = q.Id;
                        data.Add(d);
                    });
                }

            });
            services.SqlBulkCopyAdd(data);
        }
    }
}
