﻿using FCS.Common;
using FCS.Interface;
using FCS.Models;
using FCS.Models.DTO;
using FCS.Models.Entity;
using HtmlAgilityPack;
using Newtonsoft.Json;
using Quartz;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FCS.Crawler.Basketball
{
    /// <summary>
    /// 篮球联盟排名服务
    /// </summary>
    public class B_LeagueRankingJob : CommonJob, IJob
    {
        private List<B_LeagueRanking> LeagueRankingList = new List<B_LeagueRanking>();
        private List<B_LeagueRanking> CBALeagueRankingList = new List<B_LeagueRanking>();
        private List<DataItemDetail> DateItem = new List<DataItemDetail>();
        private List<B_Team> TeamList = new List<B_Team>();
        B_LeagueRanking g;

        private string cbaEventId = "";
        private string nbaEventId = "";
        public B_LeagueRankingJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            g = new B_LeagueRanking();
            cbaEventId = services.Query<B_Events>("and Name ='CBA赛事'", "").FirstOrDefault().Id;
            nbaEventId = services.Query<B_Events>("and Name ='NBA赛事'", "").FirstOrDefault().Id;
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
        }


        #region NBA
        /// <summary>
        /// 获取所有数据 NBA
        /// </summary>
        public void GetAll()
        {
            GetSqlString("");
        }
        /// <summary>
        /// 更新数据NBA
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            CommonHelper.LogBD(typeof(B_LeagueRankingJob), () =>
            {
                if (Year == "")
                {
                    Year = DateTime.Now.Year.ToString().Substring(2, 2);
                }
                GetSqlString(Year);
                GetAll_CBA(); 
            });
           
        }


        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string Year)
        {
            TeamList = services.Query<B_Team>().ToList();
            var ds = services.Query<B_Grouping>("and EventId='" + nbaEventId + "'  and Remark is not null", "", "distinct EventId,Season,Remark").ToList();
            if (Year != "")
            {
                ds = services.Query<B_Grouping>(" and EventId='" + nbaEventId + "'  and Remark is not null  and Season like '%" + Year + "%'", "", "distinct EventId,Season,Remark").ToList();
            }

            int max = ds.Count;
            int num = 0;
            ds.ForEach(async p =>
            {
                await Task.Run(() =>
                 {
                     var url = p.Remark.ToString().Replace("saiAll.do", "pai.do");
                     var eventId = p.EventId;
                     var season = p.Season;
                     GetData(url, eventId, season);

                 });
                lock (g)
                {
                    num++;
                    Monitor.Pulse(g); //完成，通知等待队列,告知已完，执行下一个。
                }

            });

            lock (g)
            {
                while (num < max)
                {
                    Monitor.Wait(g);//等待
                }
            }
            Trace.WriteLine("获取B_LeagueRanking 完结");
            try
            {
                if (Year != "")
                {
                    services.Delete<B_LeagueRanking>("and EventId='" + nbaEventId + "'  and Season like '%" + Year + "%'");
                }

                services.SqlBulkCopyAdd<B_LeagueRanking>(LeagueRankingList);
            }
            catch (Exception ex)
            {

                throw;
            }

        }
        /// <summary>
        /// 解析排行榜数据
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        private void GetData(string url, string eventId, string season)
        {
            HtmlParameterDTO dtomodel = new HtmlParameterDTO();
            dtomodel.Url = url;
            HtmlDocument doc = CommonHelper.GetHtmlHtmlDocument(dtomodel);
            if (doc.DocumentNode.InnerHtml == "Termination" || doc.DocumentNode.InnerHtml == "" || doc.DocumentNode.SelectNodes("//*[@class='paiming']") == null)
            {
                return;
            }
            var team = doc.DocumentNode.SelectNodes("//*[@class='paiming']");
            foreach (var item in team)
            {
                HtmlDocument itemdoc = new HtmlDocument();
                itemdoc.LoadHtml(item.InnerHtml);
                var trList = itemdoc.DocumentNode.SelectNodes("//table/tr").ToList();
                var TypeName = "";
                if (itemdoc.DocumentNode.InnerText.Contains("东部"))
                {
                    TypeName = "东部排行";
                }
                else
                {
                    TypeName = "西部排行";
                }
                trList.ForEach(p =>
                {
                    if (!p.InnerText.Contains("<th"))
                    {
                        HtmlDocument trdoc = new HtmlDocument();
                        trdoc.LoadHtml(p.InnerHtml);
                        if (doc.DocumentNode.SelectNodes("//td") == null)
                            return;
                        var tdList = trdoc.DocumentNode.InnerHtml.Contains("td") ? trdoc.DocumentNode.SelectNodes("//td").ToList() : null;
                        if (tdList != null && tdList.Count() > 1)
                        {
                            B_LeagueRanking ranking = new B_LeagueRanking();
                            ranking.Id = Guid.NewGuid().ToString();
                            ranking.Sort = int.Parse(tdList[0].InnerText.ToString());
                            ranking.TeamId = TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).Count() > 0 ? TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).FirstOrDefault().Id : "";
                            ranking.EventId = eventId;
                            ranking.Season = season;
                            ranking.Win = int.Parse(tdList[2].InnerText.Trim().ToString());
                            ranking.Lose = int.Parse(tdList[3].InnerText.Trim().ToString());
                            ranking.WinRate = tdList[4].InnerText.Trim().ToString();
                            ranking.WinDivide = tdList[5].InnerText.Trim().ToString();
                            ranking.ScoreAvg = tdList[6].InnerText.Trim().ToString();
                            ranking.LossAvg = tdList[7].InnerText.Trim().ToString();
                            ranking.League = tdList[8].InnerText.Trim().ToString();
                            ranking.Division = tdList[9].InnerText.Trim().ToString();
                            ranking.Home = tdList[10].InnerText.Trim().ToString();
                            ranking.Visiting = tdList[11].InnerText.Trim().ToString();
                            ranking.NearlyTen = tdList[12].InnerText.Trim().ToString();
                            ranking.Status = tdList[13].InnerText.Trim().ToString();
                            ranking.Remark = TypeName;
                            LeagueRankingList.Add(ranking);
                        }

                    }
                });
            }
        }


        #endregion


        #region CBA

        /// <summary>
        /// CBA所有数据
        /// </summary>
        public void GetAll_CBA()
        {
            GetCBALeagueRanking();
            services.Delete<B_LeagueRanking>("and EventId='" + cbaEventId + "'");
            services.SqlBulkCopyAdd<B_LeagueRanking>(CBALeagueRankingList);
        }

        /// <summary>
        /// 解析CBA js数据
        /// </summary>
        private void GetCBALeagueRanking()
        {
            int startyear = 5;
            int endtyear = DateTime.Now.AddYears(1).Year % 100;
            List<string> Years = new List<string>();
            //组装时间段
            while (startyear < endtyear)
            {
                int temp = startyear + 1;
                string Year = (startyear < 10 ? "0" + startyear.ToString() : startyear.ToString()) + "-" + (temp < 10 ? "0" + temp.ToString() : temp.ToString());
                Years.Add(Year);
                startyear++;
            }
            //获取排行榜

            foreach (var item in Years)
            {
                var url = "http://nba.nowscore.com/jsData/rank/" + item + "/s5.js";
                var doc = CommonHelper.GetHtmlHtmlDocument(new Models.DTO.HtmlParameterDTO { Url = url, Allowautoredirect = true, Timeout = 10 * 1000, Title = "lastUpdateTime" });

                var htmlResource = doc.DocumentNode.InnerHtml;
                if (htmlResource.IsEmpty() || doc.DocumentNode.InnerHtml == ConfigurationManager.AppSettings["Termination"])
                    return;
                var arrs = htmlResource.Replace("&nbsp;", " ").Split(';');
                //var arrLeague = arrs[0].Split('=')[1];
                var arrTeam = arrs[1].Split('=')[1];
                var westData = arrs[2].Split('=')[1];
                //var lastUpdateTime = arrs[3].Split('=')[1];
                //var scoreColor = arrs[4].Split('=')[1];
                var arrTeamList = JsonConvert.DeserializeObject<object[][]>(arrTeam).ToList();
                var westDataList = JsonConvert.DeserializeObject<object[][]>(westData).ToList();

                for (int i = 0; i < westDataList.Count(); i++)
                {
                    var teamName = arrTeamList.Where(o => o[0].ToString() == westDataList[i][0].ToString()).Count() > 0 ? arrTeamList.Where(o => o[0].ToString() == westDataList[i][0].ToString()).FirstOrDefault()[1] : "";
                    var teamId = services.Query<B_Team>("and Name ='" + teamName + "'", "").Count() > 0 ? services.Query<B_Team>("and Name ='" + teamName + "'", "").FirstOrDefault().Id : "";
                    B_LeagueRanking ranking = new B_LeagueRanking();
                    ranking.Id = Guid.NewGuid().ToString();
                    ranking.Sort = i + 1;
                    ranking.TeamId = teamId;
                    ranking.EventId = cbaEventId;
                    ranking.Season = item;
                    ranking.Win = int.Parse(westDataList[i][1].ToString());
                    ranking.Lose = int.Parse(westDataList[i][2].ToString());
                    ranking.WinRate = westDataList[i][3].ToString();
                    ranking.WinDivide = westDataList[i][5].ToString();
                    ranking.ScoreAvg = westDataList[i][6].ToString();
                    ranking.LossAvg = westDataList[i][7].ToString();
                    //ranking.League = westDataList[i][1].ToString();
                    //ranking.Division = westDataList[i][1].ToString();
                    ranking.Home = westDataList[i][12].ToString() + "-" + westDataList[i][13].ToString();
                    ranking.Visiting = westDataList[i][14].ToString() + "-" + westDataList[i][15].ToString();
                    ranking.NearlyTen = westDataList[i][16].ToString() + "-" + westDataList[i][17].ToString(); ;
                    ranking.Status = westDataList[i][18].ToString();
                    ranking.Remark = url;
                    CBALeagueRankingList.Add(ranking);
                }
            }

        }
        #endregion
    }
}
