﻿using System;
using FCS.Common;
using FCS.Interface;
using FCS.Models;
using FCS.Models.Entity;
using HtmlAgilityPack;
using Quartz;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using FCS.Crawler.ZCLotteryMatchs;
using System.Configuration;
using FCS.Models.DTO;
using System.Text.RegularExpressions;

namespace FCS.Crawler.Basketball
{
    /// <summary>
    /// 篮球球员统计
    /// </summary>
    public class B_PlayersStatisticsJob : CommonJob
    {
        private Dictionary<string, string> groupDict_NBA;//分组配置
        private Dictionary<string, string> typeDict_NBA;  //类型配置
        private Dictionary<string, string> typeDict_CBA;  //类型配置
        private string groupType_NBA = "_groupType";//用以得分替换字段
        private string season_NBA = "_season";//用以得分替换字段
        private string eventId;//获取赛事编码
        private List<B_Grouping> groupList;//分组
        private List<B_Team> teamList;//球队
        private List<B_Players> playerList;//球员
        private List<B_PlayersStatistics> result = new List<B_PlayersStatistics>();
        public B_PlayersStatisticsJob()
        {

            #region 篮球的页面与服务器的数据关联
            var zUrl_NBA = "http://lanqiu.zgzcw.com";//主域名
            var hUrl_NBA = "?source_league_id=1&season={0}&type={1}".FormatMe(season_NBA, groupType_NBA);//公共域名参数配置

            groupDict_NBA = new Dictionary<string, string>
            {
                {"季前赛","1" },  {"常规赛","2" },  {"季后赛","3" },
            };
            typeDict_NBA = new Dictionary<string, string>
            {
                { "得分",zUrl_NBA+"/1/{0}/{1}/playerStatistics.do".FormatMe(season_NBA,groupType_NBA)},
                { "投篮",zUrl_NBA+"/nba/playerShoot.action"+hUrl_NBA},
                { "三分",zUrl_NBA+"/nba/playerThreeShoot.action"+hUrl_NBA},
                { "罚球",zUrl_NBA+"/nba/playerPenaltyShoot.action"+hUrl_NBA},
                { "篮板",zUrl_NBA+"/nba/playerBackboard.action"+hUrl_NBA},
                { "助攻",zUrl_NBA+"/nba/playerHelp.action"+hUrl_NBA},
                { "盖帽",zUrl_NBA+"/nba/playerCover.action"+hUrl_NBA},
                { "抢断",zUrl_NBA+"/nba/playerRob.action"+hUrl_NBA},
                { "失误",zUrl_NBA+"/nba/playerMiss.action"+hUrl_NBA},
                { "犯规",zUrl_NBA+"/nba/playerFoul.action"+hUrl_NBA},
                { "两双",zUrl_NBA+"/nba/playerDouble.action"+hUrl_NBA},
                { "三双",zUrl_NBA+"/nba/playerThree.action"+hUrl_NBA}
            };
            typeDict_CBA = new Dictionary<string, string> {
                { "arrGetScore","得分"},
                { "arrShoot","投篮"},
                { "arrThreeMin","三分"},
                { "arrPunishBall","罚球"},
                { "arrBoard","篮板"},
                { "arrHelpAttack","助攻"},
                { "arrCover","盖帽"},
                { "arrRob","抢断"},
                { "arrMisplay","失误"},
                { "arrFoul","犯规"},
                { "arrDouble_2","两双"},
                { "arrDouble_3","三双"}
            };
            #endregion
        }
        public void Click()
        {
            CommonHelper.LogBD(typeof(B_PlayersStatisticsJob), () =>
            {
                Click_NBA();
                Click_CBA();
            });
          
        }
        #region NBA
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll_NBA()
        {
            //获取NBA的赛事编码
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["NBAEventName"]).ToList()[0].Id;
            GetSqlData();
            //获取NBA的分组
            groupList = services.Query<B_Grouping>(" AND (ParentId IS NULL OR ParentId='')  AND EventId='{0}'".FormatMe(eventId), "", "distinct Id,Season,Name").ToList();
            groupList.ForEach(p =>
            {
                Analysis_NBA(p);
            });
            while (true)
            {
                if (CommonHelper.ThreadsFinsh())
                    break;
            }
            services.SqlBulkCopyAdd(result);
        }


        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click_NBA(string Year = "")
        {
            //获取NBA的赛事编码
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["NBAEventName"]).ToList()[0].Id;
            GetSqlData();
            //获取NBA的分组
            groupList = services.Query<B_Grouping>(
                "  AND(ParentId IS NULL OR ParentId='')  AND EventId='{0}' AND Season LIKE '%{1}%' ".FormatMe(eventId, DateTime.Now.Year.ToString().Substring(2)), "", "distinct Id,Season,Name").ToList();
            groupList.ForEach(p =>
            {
                Analysis_NBA(p);
            });
            Thread.Sleep(3000);
            Task.WaitAll(taskList.ToArray());

            var deleteWhere = "";
            groupList.ForEach(p =>
            {
                deleteWhere += "'" + p.Id + "',";

            });
            deleteWhere = deleteWhere.ToString().Substring(0, deleteWhere.ToString().Length - 1);
            services.Delete<B_PlayersStatistics>(" AND GroupingId IN ({0})".FormatMe(deleteWhere));
            services.SqlBulkCopyAdd(result);
        }

        /// <summary>
        /// 得到数据库相关的数据——NBA
        /// </summary>

        private async Task Analysis_NBA(B_Grouping p)
        {
            foreach (var item in typeDict_NBA)
            {
                taskList.Add(Task.Run(() =>
                {
                    var url = item.Value.Replace(groupType_NBA, groupDict_NBA[p.Name]).Replace(season_NBA, p.Season);
                    var doc = CommonHelper.GetHtmlHtmlDocument(new HtmlParameterDTO { Url = url, Title = "排名", Allowautoredirect = true, });
                    var trList = doc.DocumentNode.SelectNodes(".//tr");

                    foreach (var tr in trList)
                    {
                        var td = tr.SelectNodes(".//td");
                        if (td == null || td.Count <= 0)
                            continue;

                        string playerName = td[1].SelectNodes("./a")[0].InnerText.Trim();
                        var teamName = td[2].SelectNodes("./a")[0].InnerText.Trim();

                        result.Add(new B_PlayersStatistics
                        {
                            Id = CommonHelper.GetGuid().ToString(),
                            CreateDateTime = DateTime.Now,
                            EventId = eventId,
                            GroupingId = p.Id,
                            PlayerId = (from a in playerList
                                        where a.ChineseName == playerName
                                        select a.Id).Take(1).ToJoin(),
                            PlayerName = td[1].SelectNodes("./a")[0].InnerText.Trim(),
                            TeamId = (from a in teamList
                                      where a.Name == teamName
                                      select a.Id).Take(1).ToJoin(),
                            TeamName = teamName,
                            Season = p.Season,
                            Sort = td[0].SelectNodes("./em")[0].InnerText.Trim().TryToInt32(),
                            Sum = td[3].InnerText.Trim().TryToInt32(),
                            Remark = td[1].SelectNodes("./a")[0].Attributes["href"].Value + "|" + td[2].SelectNodes("./a")[0].Attributes["href"].Value,
                            TotalShooting = td[4].InnerText,
                            HitNumber = td[5].InnerText,
                            TotalScore = item.Key == "得分" || item.Key == "篮板" ? td[6].InnerText : string.Empty,
                            AvgScore = td.Count >= 8 ? td[7].InnerText : string.Empty,
                            HitRate = td.Count == 7 ? (td[6].InnerText.Contains("%") ? td[6].InnerText : string.Empty)
                                      : td.Count == 6 ? (td[5].InnerText.Contains("%") ? td[5].InnerText : string.Empty) : string.Empty,
                            StatisticTypeName = item.Key
                        });
                    }

                }));
            }
        }
        #endregion

        #region CBA
        public void Click_CBA()
        {
            result = new List<B_PlayersStatistics>();
            //获取NBA的赛事编码
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["CBAEventName"]).ToList()[0].Id;
            GetSqlData();
            //获取NBA的分组
            groupList = services.Query<B_Grouping>(
                 "  AND(ParentId IS NULL OR ParentId='')  AND EventId='{0}' AND Season LIKE '%{1}%' ".FormatMe(eventId, DateTime.Now.Year.ToString().Substring(2)), "", "distinct Id,Season,Name").ToList();

            groupList.ForEach(p =>
            {
                Analysis_CBA(p);
            });
            Task.WaitAll(taskList.ToArray());
            var deleteWhere = "";
            groupList.ForEach(p =>
            {
                deleteWhere += "'" + p.Id + "',";

            });
            deleteWhere = deleteWhere.ToString().Substring(0, deleteWhere.ToString().Length - 1);
            services.Delete<B_PlayersStatistics>(" AND GroupingId IN ({0})".FormatMe(deleteWhere));
            services.SqlBulkCopyAdd(result);
        }

        /// <summary>
        /// 获取CBA所有数据
        /// </summary>
        public void GetAll_CBA()
        {
            //季前赛 http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason=2017-2018&SclassID=5&matchKind=3
            //常规赛 http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason=2017-2018&SclassID=5&matchKind=1
            //季后赛 http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason=2017-2018&SclassID=5&matchKind=2
            result = new List<B_PlayersStatistics>();
            //获取NBA的赛事编码
            eventId = new B_EventsJob().GetEventList().Where(p => p.Name == ConfigurationManager.AppSettings["CBAEventName"]).ToList()[0].Id;
            GetSqlData();
            //获取NBA的分组
            groupList = services.Query<B_Grouping>(
                "  AND (ParentId IS NULL OR ParentId='') AND EventId='{0}'".FormatMe(eventId), "", "distinct Id,Season,Name").ToList();
            groupList.ForEach(p =>
            {
                Analysis_CBA(p);
            });
            Task.WaitAll(taskList.ToArray());
            services.SqlBulkCopyAdd(result);
        }

        /// <summary>
        /// 异步调度
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        private async Task Analysis_CBA(B_Grouping p)
        {
            taskList.Add(Task.Run(() =>
            {
                var url = "";
                if (p.Name.Contains("季前赛"))
                    url = $" http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason={p.Season}&SclassID=5&matchKind=3";
                else if (p.Name.Contains("常规赛"))
                    url = $"http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason={p.Season}&SclassID=5&matchKind=1";
                else
                    url = $"http://nba.nowscore.com/cn/PlayerTechNor.aspx?matchSeason={p.Season}&SclassID=5&matchKind=2";
                var html = new B_GroupingJob().GetPageHtml(url, p.Season);
                //获取球队信息
                var arrTeam = Regex.Split(html, "arrTeam")[1].Split(';')[0].Replace("=", "").Trim();
                var pageTeamData = arrTeam.JsonToList<List<string>>();
                foreach (var item in typeDict_CBA)
                {
                    var data = Regex.Split(html, item.Key)[1].Split(';')[0].Replace("=", "").Trim().JsonToList<List<object>>();
                    AddData(data, pageTeamData, item.Value, p.Season, p.Id);
                }

            }));

        }

        /// <summary>
        /// result添加数据
        /// </summary>
        private void AddData(List<List<object>> data, List<List<string>> teamData, string name, string season, string groupId)
        {
            foreach (var item in data)
            {
                var teamName = new B_GamesJob().GetTeamName(teamData, item[4].ToString());
                result.Add(new B_PlayersStatistics
                {
                    Id = CommonHelper.GetGuid().ToString(),
                    CreateDateTime = DateTime.Now,
                    EventId = eventId,
                    Season = season,
                    GroupingId = groupId,
                    Sort = data.IndexOf(item) + 1,
                    StatisticTypeName = name,
                    TeamName = teamName,
                    TeamId = (from a in teamList
                              where a.Remark.Contains(item[4].ToString())
                              select a.Id).Take(1).ToJoin(),
                    PlayerName = item[1].ToString(),
                    PlayerId = (from a in playerList
                                where a.ChineseName == item[1].ToString()
                                select a.Id).Take(1).ToJoin(),
                    Sum = item[5].TryToInt32(),

                    TotalShooting = item[6].ToString(),
                    HitNumber = item.Count() >= 8 ? item[7].ToString() : string.Empty,
                    TotalScore = item.Count() >= 9 ? item[8].ToString() : string.Empty,
                    AvgScore = item.Count() >= 10 ? item[9].ToString() : string.Empty,
                    HitRate = item.Count() >= 9 ? item[8].ToString() : string.Empty
                });
            }
        }

        #endregion
        private void GetSqlData()
        {

            //获取N球队
            teamList = services.Query<B_Team>(" AND EventId='{0}'".FormatMe(eventId), "", "Id,Name,Remark").ToList();
            //获取球员
            playerList = services.Query<B_Players>(" AND b.EventId='{0}'".FormatMe(eventId), " JOIN B_Team b ON a.TeamId =b.Id", "a.Id,a.ChineseName").ToList();

        }
    }
}
