﻿using System;
using FCS.Common;
using FCS.Interface;
using FCS.Models;
using FCS.Models.Entity;
using HtmlAgilityPack;
using Quartz;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using FCS.Models.DTO;
using Newtonsoft.Json;
using System.Configuration;

namespace FCS.Crawler.Basketball
{
    /// <summary>
    /// 篮球大小盘路
    /// </summary>
    public class B_SizeBallRankingJob : CommonJob, IJob
    {
        private List<B_SizeBallRanking> SizeBallRankingList = new List<B_SizeBallRanking>();
        private List<B_SizeBallRanking> CBASizeBallRankingList = new List<B_SizeBallRanking>();
        private List<DataItemDetail> DateItem = new List<DataItemDetail>();
        private List<B_Team> TeamList = new List<B_Team>();
        B_SizeBallRanking g;
        private string cbaEventId = "";
        private string nbaEventId = "";
        public B_SizeBallRankingJob()
        {
            g = new B_SizeBallRanking();
            cbaEventId = services.Query<B_Events>("and Name ='CBA赛事'", "").FirstOrDefault().Id;
            nbaEventId = services.Query<B_Events>("and Name ='NBA赛事'", "").FirstOrDefault().Id;

        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
        }


        #region NBA
        #region 更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString("");
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            CommonHelper.LogBD(typeof(B_SizeBallRankingJob), () =>
            {
                if (Year == "")
                {
                    Year = DateTime.Now.Year.ToString().Substring(2, 2); ;
                }
                GetSqlString(Year);
                GetAll_CBA();
            });
          
        }
        #endregion
        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string Year)
        {
            TeamList = services.Query<B_Team>().ToList();
            //SizeBallRankingList = services.Query<B_SizeBallRanking>().ToList();
            var dataItem = services.GetDataItem(DataItemDetailEnum.FootBallSizeBallType);
            var ds = services.Query<B_Grouping>("and EventId='" + nbaEventId + "' and Remark is not null", "", "distinct EventId,Season,Remark").ToList();
            if (Year != "")
            {
                ds = services.Query<B_Grouping>(" and EventId='" + nbaEventId + "' and Remark is not null  and Season like '%" + Year + "%'", "", "distinct EventId,Season,Remark").ToList();
            }
            int max = ds.Count;
            int num = 0;
            ds.ForEach(async p =>
            {
                await Task.Run(() =>
                 {
                     var eventId = p.EventId;
                     var season = p.Season;
                     var url = p.Remark.ToString().Replace("saiAll.do", "1/bigSmallBall.do");
                     GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "1").First().Id.ToString());
                     url = p.Remark.ToString().Replace("saiAll.do", "2/bigSmallBall.do");
                     GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "2").First().Id.ToString());
                     url = p.Remark.ToString().Replace("saiAll.do", "3/bigSmallBall.do");
                     GetData(url, eventId, season, dataItem.Where(o => o.ItemValue == "3").First().Id.ToString());
                 });
                lock (g)
                {
                    num++;
                    Monitor.Pulse(g); //完成，通知等待队列,告知已完，执行下一个。
                }
            });
            lock (g)
            {
                while (num < max)
                {
                    Monitor.Wait(g);//等待
                }
            }
            Trace.WriteLine("获取B_SizeBallRanking 完结");
            try
            {
                if (Year != "")
                {
                    services.Delete<B_SizeBallRanking>("and EventId='" + nbaEventId + "' and Season like '%" + Year + "%'");
                }
                services.SqlBulkCopyAdd<B_SizeBallRanking>(SizeBallRankingList);
                Trace.WriteLine("B_SizeBallRanking更新完毕");
            }
            catch (Exception ex)
            {

                throw;
            }

        }

        /// <summary>
        /// 解析排行榜数据
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        private void GetData(string url, string eventId, string season, string sizeBallType)
        {
            HtmlParameterDTO dtomodel = new HtmlParameterDTO();
            dtomodel.Url = url;
            HtmlDocument doc = CommonHelper.GetHtmlHtmlDocument(dtomodel);
            if (doc.DocumentNode.InnerHtml == "Termination" || doc.DocumentNode.InnerHtml == "" || doc.DocumentNode.SelectNodes("//*[@class='paiming']") == null)
            {
                return;
            }
            var team = doc.DocumentNode.SelectNodes("//*[@class='paiming']").First();

            HtmlDocument itemdoc = new HtmlDocument();
            itemdoc.LoadHtml(team.InnerHtml);
            var trList = itemdoc.DocumentNode.SelectNodes("//table/tr").ToList();
            trList.ForEach(p =>
            {
                if (!p.InnerHtml.Contains("<th"))
                {
                    HtmlDocument trdoc = new HtmlDocument();
                    trdoc.LoadHtml(p.InnerHtml);
                    var tdList = trdoc.DocumentNode.InnerHtml.Contains("td") ? trdoc.DocumentNode.SelectNodes("//td").ToList() : null;
                    if (tdList != null && tdList.Count() > 1)
                    {
                        B_SizeBallRanking sizing = new B_SizeBallRanking();
                        sizing.Id = Guid.NewGuid().ToString();
                        sizing.Sort = int.Parse(tdList[0].InnerText.Trim().ToString());
                        sizing.SizeBallType = sizeBallType;
                        sizing.TeamId = TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).Count() > 0 ? TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).FirstOrDefault().Id : "";
                        sizing.EventId = eventId;
                        sizing.Season = season;
                        sizing.Sum = int.Parse(tdList[2].InnerText.Trim().ToString());
                        sizing.LargeNum = int.Parse(tdList[3].InnerText.Trim().ToString());
                        sizing.GoNum = int.Parse(tdList[4].InnerText.Trim().ToString());
                        sizing.SmallNum = int.Parse(tdList[5].InnerText.Trim().ToString());
                        sizing.LargePercentage = tdList[6].InnerText.Trim().ToString();
                        sizing.GoPercentage = tdList[7].InnerText.Trim().ToString();
                        sizing.SmallPercentage = tdList[8].InnerText.Trim().ToString();
                        SizeBallRankingList.Add(sizing);
                    }
                }
            });
        }

        #endregion


        #region CBA
        public void GetAll_CBA()
        {
            GetCBASizeBallRanking();
            services.Delete<B_SizeBallRanking>("and EventId='" + cbaEventId + "'");
            services.SqlBulkCopyAdd<B_SizeBallRanking>(CBASizeBallRankingList);
        }

        /// <summary>
        /// 解析CBA js数据
        /// </summary>
        private void GetCBASizeBallRanking()
        {
            int startyear = 5;
            int endtyear = DateTime.Now.AddYears(1).Year % 100;
            List<string> Years = new List<string>();
            //组装时间段
            while (startyear < endtyear)
            {
                int temp = startyear + 1;
                string Year = (startyear < 10 ? "0" + startyear.ToString() : startyear.ToString()) + "-" + (temp < 10 ? "0" + temp.ToString() : temp.ToString());
                Years.Add(Year);
                startyear++;
            }
            var dataItem = services.GetDataItem(DataItemDetailEnum.FootBallSizeBallType);
            //获取排行榜
            Task.Run(() =>
            {
                foreach (var item in Years)
                {
                    var url = "http://nba.nowscore.com/jsData/bigSmall/" + item + "/bs5.js";
                    var doc = CommonHelper.GetHtmlHtmlDocument(new Models.DTO.HtmlParameterDTO { Url = url, Allowautoredirect = true, Timeout = 10 * 1000, Title = "lastUpdateTime" });

                    var htmlResource = doc.DocumentNode.InnerHtml; if (htmlResource.IsEmpty() ||  doc.DocumentNode.InnerHtml == ConfigurationManager.AppSettings["Termination"])
                        return;
                    var arrs = htmlResource.Split(';');
                    var arrTeam = arrs[1].Split('=')[1];
                    var TotalBs = arrs[2].Split('=')[1];
                    var HomeBs = arrs[3].Split('=')[1];
                    var GuestBs = arrs[4].Split('=')[1];
                    //球队数据
                    var arrTeamList = JsonConvert.DeserializeObject<object[][]>(arrTeam).ToList();
                    //总榜数据
                    var TotalBsList = JsonConvert.DeserializeObject<object[][]>(TotalBs).ToList();
                    //主场数据
                    var HomeBsList = JsonConvert.DeserializeObject<object[][]>(HomeBs).ToList();
                    //客场数据
                    var GuestBsList = JsonConvert.DeserializeObject<object[][]>(GuestBs).ToList();

                    #region 总榜
                    var sizeBallType = dataItem.Where(o => o.ItemValue == "1").First().Id.ToString();
                    for (int i = 0; i < TotalBsList.Count(); i++)
                    {
                        var teamName = arrTeamList.Where(o => o[0].ToString() == TotalBsList[i][1].ToString()).Count() > 0 ? arrTeamList.Where(o => o[0].ToString() == TotalBsList[i][1].ToString()).FirstOrDefault()[1] : "";
                        var teamId = services.Query<B_Team>("and Name ='" + teamName + "'", "").Count() > 0 ? services.Query<B_Team>("and Name ='" + teamName + "'", "").FirstOrDefault().Id : "";
                        B_SizeBallRanking sizing = new B_SizeBallRanking();
                        sizing.Id = Guid.NewGuid().ToString();
                        sizing.Sort = int.Parse(TotalBsList[i][0].ToString());
                        sizing.SizeBallType = sizeBallType;
                        sizing.TeamId = teamId;
                        sizing.EventId = cbaEventId;
                        sizing.Season = item;
                        sizing.Sum = int.Parse(TotalBsList[i][2].ToString());
                        sizing.LargeNum = int.Parse(TotalBsList[i][3].ToString());
                        sizing.GoNum = int.Parse(TotalBsList[i][4].ToString());
                        sizing.SmallNum = int.Parse(TotalBsList[i][5].ToString());
                        sizing.LargePercentage = TotalBsList[i][6].ToString();
                        sizing.GoPercentage = TotalBsList[i][7].ToString();
                        sizing.SmallPercentage = TotalBsList[i][8].ToString();
                        CBASizeBallRankingList.Add(sizing);
                    }
                    #endregion

                    #region 主场
                    sizeBallType = dataItem.Where(o => o.ItemValue == "2").First().Id.ToString();
                    for (int i = 0; i < HomeBsList.Count(); i++)
                    {
                        var teamName = arrTeamList.Where(o => o[0].ToString() == HomeBsList[i][1].ToString()).Count() > 0 ? arrTeamList.Where(o => o[0].ToString() == HomeBsList[i][1].ToString()).FirstOrDefault()[1] : "";
                        var teamId = services.Query<B_Team>("and Name ='" + teamName + "'", "").Count() > 0 ? services.Query<B_Team>("and Name ='" + teamName + "'", "").FirstOrDefault().Id : "";
                        B_SizeBallRanking sizing = new B_SizeBallRanking();
                        sizing.Id = Guid.NewGuid().ToString();
                        sizing.Sort = int.Parse(HomeBsList[i][0].ToString());
                        sizing.SizeBallType = sizeBallType;
                        sizing.TeamId = teamId;
                        sizing.EventId = cbaEventId;
                        sizing.Season = item;
                        sizing.Sum = int.Parse(HomeBsList[i][2].ToString());
                        sizing.LargeNum = int.Parse(HomeBsList[i][3].ToString());
                        sizing.GoNum = int.Parse(HomeBsList[i][4].ToString());
                        sizing.SmallNum = int.Parse(HomeBsList[i][5].ToString());
                        sizing.LargePercentage = HomeBsList[i][6].ToString();
                        sizing.GoPercentage = HomeBsList[i][7].ToString();
                        sizing.SmallPercentage = HomeBsList[i][8].ToString();
                        CBASizeBallRankingList.Add(sizing);
                    }
                    #endregion

                    #region 客场
                    sizeBallType = dataItem.Where(o => o.ItemValue == "3").First().Id.ToString();
                    for (int i = 0; i < GuestBsList.Count(); i++)
                    {
                        var teamName = arrTeamList.Where(o => o[0].ToString() == GuestBsList[i][1].ToString()).Count() > 0 ? arrTeamList.Where(o => o[0].ToString() == GuestBsList[i][1].ToString()).FirstOrDefault()[1] : "";
                        var teamId = services.Query<B_Team>("and Name ='" + teamName + "'", "").Count() > 0 ? services.Query<B_Team>("and Name ='" + teamName + "'", "").FirstOrDefault().Id : "";
                        B_SizeBallRanking sizing = new B_SizeBallRanking();
                        sizing.Id = Guid.NewGuid().ToString();
                        sizing.Sort = int.Parse(GuestBsList[i][0].ToString());
                        sizing.SizeBallType = sizeBallType;
                        sizing.TeamId = teamId;
                        sizing.EventId = cbaEventId;
                        sizing.Season = item;
                        sizing.Sum = int.Parse(GuestBsList[i][2].ToString());
                        sizing.LargeNum = int.Parse(GuestBsList[i][3].ToString());
                        sizing.GoNum = int.Parse(GuestBsList[i][4].ToString());
                        sizing.SmallNum = int.Parse(GuestBsList[i][5].ToString());
                        sizing.LargePercentage = GuestBsList[i][6].ToString();
                        sizing.GoPercentage = GuestBsList[i][7].ToString();
                        sizing.SmallPercentage = GuestBsList[i][8].ToString();
                        CBASizeBallRankingList.Add(sizing);
                    }
                    #endregion
                }
            });

        }
        #endregion
    }
}
